options(contrasts=c("contr.sum","contr.poly"))
options(warn=-1)

options(scipen=999)
options(width=256)

library("gdata")


library("ROCR")
library("pROC")
library("randomForest")
library("caret")

library("e1071")

library("pracma")
library("tcltk")

# clear all
rm(list=ls())


algorithm = 1
rets <- c("threshold", "sensitivity", "specificity", "ppv", "npv", "accuracy")
policy = "omit"
n_boot = 4000
thresholds = "local maxima"

min_importance = 1


d = read.csv("./database.csv",sep=",")
d$class = factor(d$class)

X = d[,c(3:(dim(d)[2]))]

n_show = 20

writeLines("Using ROC AUC as feature importance metric...")

rocs = data.frame(Variable=character(0), auc=numeric(0), threshold=numeric(0), sens=numeric(0), spec=numeric(0), ppv=numeric(0), npv=numeric(0), acc=numeric(0))

j=1
while(j<=dim(X)[2]){
	p_roc <- roc(controls=X[,j][d$class=="HC"],cases=X[,j][d$class=="RBD"],quiet=TRUE)
	res <- coords(p_roc, x="best", input="threshold", ret=rets, best.policy = policy)
	rocs[j,] <- data.frame(Variable = colnames(X)[j], auc=p_roc$auc[[1]], threshold=res$threshold, sens=res$sensitivity,
					spec=res$specificity, ppv=res$ppv, npv=res$npv, acc=res$accuracy)
	j = j+1
}

i_imp = order(rocs$auc,decreasing=TRUE)
bvars = colnames(X[,i_imp])
print(rocs[i_imp,])

best = bvars[1]

imp <- data.frame(Variable = bvars,importance=round(rocs$auc[i_imp],2))
saveRDS(imp,file="importance_AUC.RDS")

# NOW CYCLE ALL in a leave-one-out fashion, using (best) variable with highest AUC
subj = c(1:dim(X)[1])
RESP = zeros(dim(X)[1],1)
THRES = zeros(dim(X)[1],1)
for (j in subj){
	x_new = X[-j,best]
	y_new = d$class[-j]
	p_roc <- roc(controls=x_new[y_new=="HC"],cases=x_new[y_new=="RBD"],quiet=TRUE)
	res <- coords(p_roc, x="best", input="threshold", ret=rets, best.policy = policy)
	THRES[j] = res$threshold
	if(X[j,best]<=THRES[j]){
		RESP[j] = 1
	}else{
		RESP[j] = 0
	}
}

y_ref = as.numeric(d$class)-1
roc_c = roc(y_ref,RESP,quiet=TRUE)
cc = confusionMatrix(as.factor(RESP),as.factor(y_ref),positive="1")
auc = roc_c$auc
res <- coords(roc_c, x="best", input="threshold", ret=rets, best.policy = policy)
acc = cc$overall[1]
acc_low = cc$overall[3]
acc_up = cc$overall[4]
sens = res$sensitivity
spec = res$specificity
fprintf("ROC-AUC %s, accuracy=%.2f(%.2f,%.2f), sens=%.2f, spec=%.2f, AUC=%.2f\n",best,acc,acc_low,acc_up,sens,spec,auc)
save(RESP, y_ref, roc_c, best, acc, file="roc_auc_simple_model.rdata")
