## generate plots
library("pROC")
library("ggplot2")
library("gridExtra")
library("ggpubr")
library("caret")

cal_data = data.frame(class=as.factor(as.numeric(d$class)-1))

#load models


#LR GLM
load("model_glm_2vars.rdata")
x = as.data.frame(d[,best_vars])
colnames(x) <- best_vars
cal_data$LR = predict(mdl, x, type="response")
#cal_data$LR = PROBS

 
#ROC-AUC
load("model_glm_1vars.rdata")
x = as.data.frame(d[,best_vars])
colnames(x) <- best_vars
cal_data$AUC = predict(mdl, x, type="response")
#cal_data$AUC = PROBS

#RF
load("model_rf_mean_5vars.rdata")
model_rf = out
cal_data$RF = predict(model_rf, d[,best_vars], type="prob")[,"RBD"]

#XGBoost
load("model_XGB_mean_17vars.RData")
model_xgb = xgb_model
cal_data$XGBOOST = predict(model_xgb, d[,best_vars], type="prob")[,"RBD"]


write.csv(cal_data,file="calib_data.csv",row.names=FALSE)
# use calibration_curve(...) from scikit.learn to compute calibration curve
# run "get_calibration_curves.py"
system("python3 get_calibration_curves.py")
# and load csv
cal_prob = read.csv("./calib_prob.csv",sep=",")
cal_prob$method = factor(cal_prob$method, levels=c("TIw","LR","RF","XGBoost"))

#my_palette = c("#00F055","orange","red","blue")
my_palette = c("#F8766D", "#7CAE00", "#00BFC4", "#C77CFF")

g_cal <- ggplot(data=cal_prob, aes(y=true_class,x=pred_prob,group=method,colour=method)) +
	geom_line(size=1.5) + geom_point(shape=18, size=7) + geom_abline(slope=1,linetype="dashed",size=1) +
	scale_colour_manual(name="method",values = c(TIw=my_palette[1],LR=my_palette[2],RF=my_palette[3],XGBoost=my_palette[4])) + 
	labs(x = "predicted probability", y = "true probability in each bin") + theme_linedraw() +
	theme(text = element_text(size=12,colour = "black"),axis.text = element_text(size=12,colour="black"),axis.title=element_text(size=14,colour="black"))
print(g_cal)
